from fastapi import APIRouter, Depends, HTTPException
from sqlmodel import Session, select, delete
from backend.database import get_session
from backend.routes.auth import get_current_user
from backend.models.models import Order, OrderItem, User, Product
from backend.schemas.order import OrderCreate, OrderRead

router = APIRouter()


@router.post("/", response_model=OrderRead)
def create_order(
    order_data: OrderCreate,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    total_price = 0
    for item in order_data.items:
        product = session.get(Product, item.product_id)
        if not product or product.stock < item.quantity:
            raise HTTPException(
                status_code=400, detail=f"Product {item.product_id} is out of stock"
            )
        total_price += item.quantity * product.price

    new_order = Order(
        user_id=current_user.id, shop_id=order_data.shop_id, total_price=total_price
    )
    session.add(new_order)
    session.commit()
    session.refresh(new_order)

    for item in order_data.items:
        order_item = OrderItem(
            order_id=new_order.id,
            product_id=item.product_id,
            quantity=item.quantity,
            price=product.price,
        )
        session.add(order_item)
        product.stock -= item.quantity

    session.commit()
    return new_order


# list all orders associated with the current user
@router.get("/list", response_model=list[OrderRead])
def list_orders(
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    orders = session.exec(select(Order).where(Order.user_id == current_user.id)).all()
    return orders


# retrieve a specific order, only the owner can access
@router.get("/{order_id}", response_model=OrderRead)
def get_order(
    order_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    order = session.get(Order, order_id)
    if not order or order.user_id != current_user.id:
        raise HTTPException(status_code=404, detail="Order not found")
    return order


# update order status
@router.put("/status/{order_id}", response_model=OrderRead)
def update_order(
    order_id: int,
    status: str,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    order = session.get(Order, order_id)
    if not order or order.user_id != current_user.id:
        raise HTTPException(status_code=404, detail="Order not found or unauthorized")
    if order.status != "pending" or status != "completed":
        raise HTTPException(
            status_code=400,
            detail="Order can only be updated from pending to completed",
        )
    order.status = status
    session.commit()
    session.refresh(order)
    return order


# Delete order
@router.delete("/{order_id}")
def delete_order(
    order_id: int,
    session: Session = Depends(get_session),
    current_user: User = Depends(get_current_user),
):
    order = session.get(Order, order_id)
    if not order or (
        order.user_id != current_user.id and current_user.role != "shop_owner"
    ):
        raise HTTPException(status_code=404, detail="Order not found or unauthorized")

    # Delete related order items first
    session.exec(delete(OrderItem).where(OrderItem.order_id == order.id))

    # Now delete the order
    session.delete(order)
    session.commit()

    return {"message": "Order deleted successfully"}
